/*
 * Egomania: alias.c: Command aliasing.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

struct ego_alias {
	char * f_name;
	char * f_target;
};

static struct ego_alias * _ego_alias_list;
static uint4 _ego_alias_count;
static uint1 _ego_alias_init = 0;

char ** _ego_argv (char *, uint1);

static void _ego_alias_start (void)
{
	struct ego_alias * ptr;
	ptr = (struct ego_alias *) malloc_m(sizeof(struct ego_alias) * 2);
	if (!ptr)
		return;
	_ego_alias_list = ptr;
	_ego_alias_count = 0;
	_ego_alias_init = 1;
}

char * ego_alias_get (char * name)
{
	register int i;
	if (!_ego_alias_init)
		return NULL;
	for (i = 0; i < _ego_alias_count; i++)
		if (_ego_alias_list[i].f_name != NULL)
			if (!strcmp(_ego_alias_list[i].f_name, name))
				return strdup(_ego_alias_list[i].f_target);
	return NULL;
}

int ego_alias_set (char * name, char * target)
{
	register int i;
	struct ego_alias * ptr;
	if (!_ego_alias_init)
		_ego_alias_start();
	for (i = 0; i < _ego_alias_count; i++)
		if (_ego_alias_list[i].f_name != NULL)
			if (!strcmp(_ego_alias_list[i].f_name, name))
				break;
	if (i == _ego_alias_count) {
		ptr = (struct ego_alias *) realloc_m((struct ego_alias *) _ego_alias_list, sizeof(struct ego_alias) * (i + 2));
		if (!ptr)
			return -1;
		_ego_alias_list = ptr;
		_ego_alias_count++;
		_ego_alias_list[_ego_alias_count].f_name = NULL;
		_ego_alias_list[_ego_alias_count].f_target = NULL;
	}
	_ego_alias_list[i].f_name = ego_strdup(name, MARK);
	_ego_alias_list[i].f_target = ego_strdup(target, MARK);
	if ((!_ego_alias_list[i].f_name) || (!_ego_alias_list[i].f_target)) {
		unmark(_ego_alias_list[i].f_name);
		unmark(_ego_alias_list[i].f_target);
		return -1;
	}
	return 0;
}

int ego_alias_put (char * str)
{
	char ** tmp;
	if (!_ego_alias_init)
		_ego_alias_start();
	tmp = ego_strexp(str, '=', 2);
	if (!tmp)
		return -1;
	return ego_alias_set(tmp[0], tmp[1]);
}

int ego_alias_unset (char * name)
{
	register int i;
	for (i = 0; i < _ego_alias_count; i++)
		if (_ego_alias_list[i].f_name != NULL)
			if (!strcmp(_ego_alias_list[i].f_name, name))
				break;
	if (i == _ego_alias_count)
		return -1;
	unmark(_ego_alias_list[i].f_name);
	unmark(_ego_alias_list[i].f_target);
	_ego_alias_list[i].f_name = NULL;
	_ego_alias_list[i].f_target = NULL;
	return 0;
}

char ** ego_alias_parse (char * str)
{
	char ** ret;
	char * tmp;
	tmp = ego_alias_get(str);
	if (tmp != NULL) {
		ret = _ego_argv(tmp, 0);
		if (!ret)
			return NULL;
		return ret;
	}
	return NULL;
}

void ego_alias_dump (void)
{
	register int i;
	for (i = 0; i < _ego_alias_count; i++) {
		if (_ego_alias_list[i].f_name)
			printf("%-15s %s\n", _ego_alias_list[i].f_name, _ego_alias_list[i].f_target);
	}
}
