/*
 * Egomania: alloc.c: Memory allocater wrappers; dumb garbage collection.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

/* ego.h defines these to our functions... */
#undef malloc
#undef realloc

struct ego_memstack {
	uint1 p_mark;
	uint4 p_size;
	uint4 p_addr;
};

static struct ego_memstack _ego_alloc_stack[EGO_ALLOC_MAX];
static uint1 _ego_alloc_init = 0;

static void _ego_alloc_start (void)
{
	memset(_ego_alloc_stack, 0, sizeof(struct ego_memstack) * EGO_ALLOC_MAX);
	_ego_alloc_init = 1;
}

static int _ego_alloc_find (void * ptr)
{
	register int i;
	for (i = 0; i < EGO_ALLOC_MAX; i++)
		if (_ego_alloc_stack[i].p_addr == (uint4) ptr)
			return i;
	return -1;
}

void * ego_malloc (uint4 size, uint1 mark)
{
	void * ptr;
	int i;
	if (_ego_alloc_init == 0)
		_ego_alloc_start();
	ptr = malloc(size + 2);
	if (!ptr)
		return NULL;
	i = _ego_alloc_find(NULL);
	if (i == -1) {
		/* reap it here... */
		free(ptr);
		return NULL;
	}
	_ego_alloc_stack[i].p_addr = (uint4) ptr;
	_ego_alloc_stack[i].p_mark = mark;
	_ego_alloc_stack[i].p_size = size + 2;
	memset(ptr, 0, size + 2);
	return ptr;
}

void * ego_realloc (void * ptr, uint4 size, uint1 mark)
{
	void * tmp;
	int i;
	if (_ego_alloc_init == 0)
		_ego_alloc_start();
	if (!ptr)
		return ego_malloc(size, mark);
	i = _ego_alloc_find(ptr);
	if (i == -1)
		return NULL;
	tmp = realloc(ptr, size + 2);
	if (!tmp)
		return NULL;
	_ego_alloc_stack[i].p_addr = (uint4) tmp;
	_ego_alloc_stack[i].p_mark = mark;
	_ego_alloc_stack[i].p_size = size + 2;
	return tmp;
}

int ego_set_mark (void * ptr, uint1 mark)
{
	int i;
	i = _ego_alloc_find(ptr);
	if (i == -1)
		return -1;
	_ego_alloc_stack[i].p_mark = mark;
	return 0;
}

void ego_alloc_reap (void)
{
	register int i;
	for (i = 0; i < EGO_ALLOC_MAX; i++)
		if ((_ego_alloc_stack[i].p_addr) && (_ego_alloc_stack[i].p_mark != MARK)) {
			free((void *) _ego_alloc_stack[i].p_addr);
			_ego_alloc_stack[i].p_addr = 0;
			_ego_alloc_stack[i].p_mark = 0;
			_ego_alloc_stack[i].p_size = 0;
		}
}

char * ego_strdup (char * str, uint1 mark)
{
	char * ret;
	ret = ego_malloc(strlen(str) + 3, mark);
	if (!ret)
		return NULL;
	strcpy(ret, str);
	return ret;
}
