/*
 * Egomania: argvec.c: Take a command string and parse it into an argv vector.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

int ego_argv_ifs (int ch)
{
	char * ifs;
	register int i;
	ifs = ego_config_get("/user/environment/ifs");
	if (!ifs)
		return -1;
	for (i = 0; ifs[i] != '\0'; i++)
		if (ch == ifs[i])
			return 0;
	return -1;
}

char ** _ego_argv (char * str, uint1 flag)
{
	int zargc = 0;
	int ch;
	int lch = 0;
	int quote = 0;
	int len;
	int i;
	int ff = 0;
	char * stmp;
	char * tmp;
	char * s;
	char ** zargv;
	char ** ztmp;
	char ** wcards = NULL;
	if (!str)
		return NULL;
	str = ego_var_parse(str);
	if (!str)
		return NULL;
	len = strlen(str);
	zargv = (char **) malloc(sizeof(char *) * 2);
	if (!zargv)
		return NULL;
	tmp = malloc(len + 2);
	if (!tmp)
		return NULL;
	for (stmp = str; *stmp != '\0';) {
		memset(tmp, 0, len + 2);
		s = tmp;
		lch = 0;
		for (i = 0; ((ch = *stmp++) != '\0') && (i < len); i++) {
			if ((!ego_argv_ifs(ch)) && (lch == '\0'))
				continue;
			else if (((ch == '\'') || (ch == '"') || (ch == '`')) && (lch != '\\')) {
				if (quote == ch)
					quote = 0;
				else if (quote == 0)
					quote = ch;
				else
					*s++ = ch;
			} else if ((quote == 0) && (lch != '\\') && (!ego_argv_ifs(ch)))
				break;
			else if ((ch != '\\') || ((ch == '\\') && (lch == '\\')))
				*s++ = ch;
			lch = ch;
		}
		*s = '\0';
		if (quote != 0) {
			printf("Egomania: Parse error looking for matching %c.\n", quote);
			return NULL;
		}
		if (strlen(tmp) < 1)
			continue;
		// do some wildcard stuffage here...
		if ((flag != 0) && (ff == 0) && (strchr(tmp, '=') == NULL)) {
			ff = 1;
			wcards = ego_alias_parse(tmp);
		}// else if (ff == 0)
//			wcards = ego_wcard_parse(tmp);
		if (wcards != NULL) {
			for (i = 0; wcards[i] != NULL; i++) {
				ztmp = (char **) realloc((char **) zargv, sizeof(char *) * (zargc + 2));
				if (!ztmp)
					return NULL;
				zargv = ztmp;
				zargv[zargc] = strdup(wcards[i]);
				if (!zargv[zargc])
					return NULL;
				zargc++;
			}
		} else {
			ztmp = (char **) realloc((char **) zargv, sizeof(char **) * (zargc + 2));
			if (!ztmp)
				return NULL;
			zargv = ztmp;
			zargv[zargc] = malloc(strlen(tmp) + 2);
			if (!zargv[zargc])
				return NULL;
			strncpy(zargv[zargc], tmp, strlen(tmp));
			zargc++;
		}
	}
	zargv[zargc] = NULL;
	return zargv;
}

int ego_argv (char * str, int * argc, char *** argv)
{
	char ** zargv;
	int zargc = 0;
	*argc = 0;
	*argv = NULL;
	zargv = _ego_argv(str, 1);
	if (!zargv)
		return -1;
	for (; zargv[zargc] != NULL; zargc++);
	*argc = zargc;
	*argv = zargv;
	return 0;
}
