/*
 * Egomania: command.c: Interpreter frontend; ties the shell together.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

static int _ego_command (char * str)
{
	char ** argv;
	int argc;
	int code;
	if (strlen(str) < 1)
		return -1;
	/* at the advent of pipes, this will get more complicated... */
	if (ego_argv(str, &argc, &argv) != 0)
		return -1;
	code = ego_exec_internal(argc, argv);
	if (code == 0x200)
		return ego_exec_external(argc, argv);
#ifdef EGO_USER_UTILS
	else if (code == 0x400)
		return ego_exec_utils(argc, argv);
#endif
	return code;
}

int ego_command (char * str)
{
	char ** plist;
	int pipes[2][2];
	int code = 0;
	register int i;
	if (strlen(str) < 1)
		return -1;
	if (strchr(str, '|') != NULL) {
		plist = ego_strexp(str, '|', 0);
		if (!plist)
			return -1;
		if ((pipe(pipes[0]) != 0) || (pipe(pipes[1]) != 0)) {
			printf("Egomania: Could not create pipelines: %s\n", strerror(errno));
			return -1;
		}
		for (i = 0; plist[i] != NULL; i++) {
			if ((i % 2) == 0) {
				ego_config_set("/core/pipe/input", (int *) pipes[1][0]);
				ego_config_set("/core/pipe/output", (int *) pipes[0][1]);
			} else {
				ego_config_set("/core/pipe/input", (int *) pipes[0][0]);
				ego_config_set("/core/pipe/output", (int *) pipes[1][1]);
			}
			ego_config_set("/core/pipe/use", "YUP");
			if (plist[i + 1] == NULL) {
				ego_config_set("/core/pipe/output", NULL);
				ego_config_set("/core/pipe/use", NULL);
			}
			if (i == 0)
				ego_config_set("/core/pipe/input", NULL);
			code = _ego_command(plist[i]);
		}
		close(pipes[0][0]);
		close(pipes[0][1]);
		close(pipes[1][0]);
		close(pipes[1][1]);
		ego_config_set("/core/pipe/use", NULL);
		ego_config_set("/core/pipe/input", NULL);
		ego_config_set("/core/pipe/output", NULL);
		return code;
	}
	return _ego_command(str);
}
