/*
 * Egomania: config.c: Central configuration system.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

struct ego_config {
	char * f_name;
	void * f_addr;
};

static struct ego_config _ego_config_list[EGO_CONFIG_MAX];
static uint1 _ego_config_init = 0;

static void _ego_config_start (void)
{
	memset(_ego_config_list, 0, sizeof(struct ego_config) * EGO_CONFIG_MAX);
	_ego_config_init = 1;
}

static int _ego_config_find (char * name)
{
	register int i;
	for (i = 0; i < EGO_CONFIG_MAX; i++)
		if ((_ego_config_list[i].f_name == NULL) && (name == NULL))
			return i;
		else if ((_ego_config_list[i].f_name) && (name != NULL)) {
			if (!strcmp(_ego_config_list[i].f_name, name))
				return i;
		}	
	return -1;
}

int ego_config_set (char * name, void * addr)
{
	int i;
	if (!_ego_config_init)
		_ego_config_start();
	i = _ego_config_find(name);
	if (i == -1) {
		i = _ego_config_find(NULL);
		if (i == -1)
			return -1;
	}
	_ego_config_list[i].f_name = ego_strdup(name, MARK);
	if (!_ego_config_list[i].f_name)
		return -1;
	_ego_config_list[i].f_addr = addr;
	/* addr is now a global, so we don't want the reaper killing it */
	mark(addr);
	return 0;
}

void * ego_config_get (char * name)
{
	int i;
	if (!_ego_config_init)
		return NULL;
	i = _ego_config_find(name);
	if (i == -1)
		return NULL;
	return _ego_config_list[i].f_addr;
}

void ego_config_dump (void)
{
	register int i;
	printf("###  %-30s   Value/Address\n\n", "Name");
	for (i = 0; i < EGO_CONFIG_MAX; i++)
		if (_ego_config_list[i].f_name != NULL)
			printf("%3d: %-30s   0x%08x\n", i, _ego_config_list[i].f_name, (unsigned int) _ego_config_list[i].f_addr);
}
