/*
 * Egomania: ego.h: Master header for Egomania
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#ifndef _EGOMANIA_INCLUDE_H_
#define _EGOMANIA_INCLUDE_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include <sys/wait.h>
#include <signal.h>
#include <pwd.h>

#define EGOMANIA_VERSION		"1.8.0"
#define EGO_ALLOC_MAX		0x200
#define EGO_CONFIG_MAX		0x50
#define EGO_COMMAND_MAX		393216

#define MARK				0x11
#define UNMARK				0x00

#undef malloc
#undef realloc
#undef malloc_m
#undef realloc_m
#undef mark
#undef unmark
#undef strdup

#define malloc(x)			ego_malloc(x, UNMARK)
#define realloc(x,l)		ego_realloc(x, l, UNMARK)
#define malloc_m(x)			ego_malloc(x, MARK)
#define realloc_m(x,y)		ego_realloc(x, y, MARK)
#define mark(x)			ego_set_mark(x, MARK)
#define unmark(x)			ego_set_mark(x, UNMARK)
#define strdup(x)			ego_strdup(x, UNMARK)

typedef unsigned char		uint1;
typedef unsigned short		uint2;
typedef unsigned long		uint4;

void * ego_malloc (uint4, uint1);
void * ego_realloc (void *, uint4, uint1);
int ego_set_mark (void *, uint1);
void ego_alloc_reap (void);
char * ego_strdup (char *, uint1);

int ego_config_set (char *, void *);
void * ego_config_get (char *);
void ego_config_dump (void);

char * ego_alias_get (char *);
int ego_alias_set (char *, char *);
int ego_alias_put (char *);
int ego_alias_unset (char *);
char ** ego_alias_parse (char *);
void ego_alias_dump (void);

int ego_argv_ifs (int);
int ego_argv (char *, int *, char ***);

char ** ego_strexp (char *, char, int);

char * ego_var_parse (char *);

int ego_command (char *);

int ego_exec_internal (int, char **);
int ego_exec_external (int, char **);

char * ego_prompt (void);

#endif
