/*
 * Egomania: egomania.c: The main() for Egomania.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

static void _ego_main_setenv (void)
{
	struct passwd * pw;
	pw = getpwuid(getuid());
	if (pw) {
		ego_config_set("/user/environment/uid", (int *) pw->pw_uid);
		ego_config_set("/user/environment/gid", (int *) pw->pw_gid);
		ego_config_set("/user/environment/name", pw->pw_name);
		ego_config_set("/user/environment/home", pw->pw_dir);
	} else {
		ego_config_set("/user/environment/uid", (int *) 1000);
		ego_config_set("/user/environment/gid", (int *) 1000);
		ego_config_set("/user/environment/name", "luser");
		ego_config_set("/user/environment/home", "/");
	}
}

int main (int argc, char ** argv)
{
	char * cmd;
	ego_config_set("/core/pipe/stdout", (int *) STDOUT_FILENO);
	ego_config_set("/core/pipe/stderr", (int *) STDERR_FILENO);
	ego_config_set("/core/pipe/input", NULL);
	ego_config_set("/core/pipe/output", NULL);
	ego_config_set("/core/pipe/use", NULL);
	ego_config_set("/user/environment/prompt", "\\u:\\d> ");
	ego_config_set("/user/environment/ifs", " \t\n");
	ego_alias_set("quit", "exit");
	ego_alias_set("chdir", "cd");
	_ego_main_setenv();
	for (;;) {
		ego_alloc_reap();
		printf("%s", ego_prompt());
		cmd = malloc(EGO_COMMAND_MAX + 2);
		if (!cmd)
			break;
		fgets(cmd, EGO_COMMAND_MAX, stdin);
		if (cmd[strlen(cmd) - 1] == '\n')
			cmd[strlen(cmd) - 1] = '\0';
		ego_command(cmd);
	}
	exit(1);
}
