/*
 * Egomania: exece.c: External program executor.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

#ifndef WCOREDUMP
#	define WCOREDUMP(x)		0
#endif

int ego_exec_external (int argc, char ** argv)
{
	int code = 0;
	int pid;
	int stat;
	if (argc < 1)
		return -1;
	signal(SIGINT, SIG_IGN);
	signal(SIGQUIT, SIG_IGN);
	switch (pid = fork()) {
		case -1: return -1;
		case 0:
			signal(SIGINT, SIG_DFL);
			signal(SIGQUIT, SIG_DFL);
			if (ego_config_get("/core/pipe/input") != NULL)
				dup2((int) ego_config_get("/core/pipe/input"), STDIN_FILENO);
			if (ego_config_get("/core/pipe/output") != NULL)
				dup2((int) ego_config_get("/core/pipe/output"), STDOUT_FILENO);
			execvp(argv[0], argv);
			printf("Egomania: Execution of '%s' failed: %s\n", argv[0], strerror(errno));
			_exit(127);
		default:
			ego_config_set("/core/process/pid", (int *) pid);
			ego_config_set("/core/process/name", argv[0]);
//			if (!ego_config_get("/core/pipe/use"))
				waitpid(pid, &stat, WUNTRACED);
	}
	signal(SIGQUIT, SIG_DFL);
	signal(SIGINT, SIG_DFL);
	ego_config_set("/core/process/pid", NULL);
	ego_config_set("/core/process/name", NULL);
	if (WIFSIGNALED(stat)) {
		if ((WTERMSIG(stat) != SIGINT) && (WTERMSIG(stat) != SIGSTOP) &&
		    (WTERMSIG(stat) != SIGTSTP)) {
			if (WCOREDUMP(stat))
				printf("Egomania: Process %s (%d) exited with signal %d (core dumped)\n", argv[0], pid, WTERMSIG(stat));
			else
				printf("Egomania: Process %s (%d) exited with signal %d\n", argv[0], pid, WTERMSIG(stat));
		}
		code = 255;
	} else if (WIFEXITED(stat))
		code = WEXITSTATUS(stat);
	return code;
}
