/*
 * Egomania: execi.c: Builtin commands.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

struct ego_command {
	char * arg0;
	int (* exec) (int argc, char ** argv);
	void (* help) (void);
};

static int _ego_cmd_cd (int argc, char ** argv)
{
	char * dir;
	char tmp[1024];
	if (argc > 2) {
		return -1;
	}
	if (argc < 2)
		dir = ego_config_get("/user/environment/home");
	else
		dir = argv[1];
	memset(tmp, 0, 1024);
	getcwd(tmp, 1024);
	if (chdir(dir) != 0) {
		printf("Egomania: Could not change to %s: %s\n", dir, strerror(errno));
		chdir(tmp);
		return -1;
	}
	memset(tmp, 0, 1024);
	getcwd(tmp, 1024);
	setenv("PWD", tmp, 1);
	return 0;
}

static int _ego_cmd_exit (int argc, char ** argv)
{
	exit(0);
}

static int _ego_cmd_set (int argc, char ** argv)
{
	extern char ** environ;
	register int i;
	char ** tmp;
	if (argc < 2) {
		for (i = 0; environ[i] != NULL; i++) {
			if (strchr(environ[i], '=') != NULL) {
				tmp = ego_strexp(environ[i], '=', 2);
				if (!tmp)
					return -1;
				printf("%-15s %s\n", tmp[0], tmp[1]);
			} else
				printf("%s\n", environ[i]);
		}
		return 0;
	}
	for (i = 1; i < argc; i++)
		putenv(ego_strdup(argv[i], MARK));
	return 0;
}

static int _ego_cmd_unset (int argc, char ** argv)
{
	register int i;
	if (argc < 2) {
		return -1;
	}
	for (i = 1; i < argc; i++) {
		if (unsetenv(argv[i]) != 0) {
			printf("Egomania: Non-existant variable: %s\n", argv[i]);
			return -1;
		}
	}
	return 0;
}

static int _ego_cmd_alias (int argc, char ** argv)
{
	register int i;
	if (argc < 2) {
		ego_alias_dump();
		return 0;
	}
	for (i = 1; i < argc; i++)
		ego_alias_put(argv[i]);
	return 0;
}

static int _ego_cmd_unalias (int argc, char ** argv)
{
	register int i;
	if (argc < 2) {
		return -1;
	}
	for (i = 1; i < argc; i++) {
		if (ego_alias_unset(argv[i])) {
			printf("Egomania: Non-existant alias %s\n", argv[i]);
			return -1;
		}
	}
	return 0;
}

static int _ego_cmd_cfd (int argc, char ** argv)
{
	ego_config_dump();
	return 0;
}

static struct ego_command _ego_command_list[] = {
	{ "cd", _ego_cmd_cd, NULL },
	{ "exit", _ego_cmd_exit, NULL },
	{ "set", _ego_cmd_set, NULL },
	{ "unset", _ego_cmd_unset, NULL },
	{ "alias", _ego_cmd_alias, NULL },
	{ "unalias", _ego_cmd_unalias, NULL },
	{ "_cfgdmp", _ego_cmd_cfd, NULL },
	{ NULL, NULL, NULL }
};

int ego_exec_internal (int argc, char ** argv)
{
	register int i;
	for (i = 0; _ego_command_list[i].arg0 != NULL; i++)
		if (!strcmp(_ego_command_list[i].arg0, argv[0]))
			return _ego_command_list[i].exec(argc, argv);
	return 0x200;
}
