/*
 * Egomania: vars.c: Variable parsing.
 * Copyright (C) 2003 Kyle Donaldson <gile@gilex.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "ego.h"

static int _ego_var_invalid (int ch)
{
	char * alist = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_:-@\0";
	register int i;
	for (i = 0; alist[i] != '\0'; i++)
		if (alist[i] == ch)
			return 1;
	return 0;
}

char * ego_var_parse (char * str)
{
	char * ret;
	char * tmp;
	char * s;
	char * st;
	char * t;
	int ch;
	int lch = 0;
	int ct = 0;
	int slen;
	int quote = 0;
//	int brk;
	int i;
	if (!str)
		return NULL;
	slen = strlen(str);
	ret = malloc(slen + 2);
	if (!ret)
		return NULL;
	s = ret;
	for (st = str; (ch = *st++) != '\0'; lch = ch) {
		if ((ct + 2) == slen) {
			tmp = realloc(ret, slen + strlen(str) + 2);
			if (!tmp)
				return NULL;
			ret = tmp;
			slen += strlen(str);
			s = ret + ct;
		}
		if ((lch != '\\') && (ch == '\'')) {
			if (quote == ch)
				quote = 0;
			else if (quote == 0)
				quote = ch;
		}
		if ((lch != '\\') && (quote == 0) && (ch == '$')) {
			tmp = malloc(slen + 2);
			if (!tmp)
				return NULL;
			t = tmp;
			for (i = 0; ((ch = *st++) != '\0') && (i < slen); i++) {
				if ((lch != '\\') && (_ego_var_invalid(ch) == 0)) {
					ch = *--st;
					break;
				}
				*t++ = ch;
			}
			*t = 0;
			tmp = getenv(tmp);
			if (tmp) {
				if ((ct + strlen(tmp)) >= slen) {
					t = realloc(ret, strlen(tmp) + slen + 2);
					if (!t)
						return NULL;
					slen += strlen(tmp);
					ret = t;
					s = ret + ct;
				}
				t = tmp;
				for (i = strlen(tmp); i > 0; i--) {
					*s++ = *t++;
					ct++;
				}
			}
		} else {
			*s++ = ch;
			ct++;
		}
	}
	*s = 0;
	return ret;
}
